<?php
/**
 * SocialEngine
 *
 * @category   Engine
 * @package    Engine_Service_Iframely_Host
 * @copyright  Copyright 2006-2010 Webligo Developments
 * @license    http://www.socialengine.com/license/
 * @version    $Id: Exception.php 9747 2012-07-26 02:08:08Z john $
 */

/**
 * @category   Engine
 * @package    Engine_Service_Iframely_Host
 * @copyright  Copyright 2006-2010 Webligo Developments
 * @license    http://www.socialengine.com/license/
 */
class Sitegifplayer_Service_GiphyAPI extends Zend_Service_Abstract
{

  /**
   * The accept header
   *
   * @var string
   */
  protected $_accept = 'application/json';

  /**
   * The base URL for host iframely
   *
   * @var string
   */
  protected $_baseUrl = 'http://api.giphy.com/v1/gifs/';

  /**
   * Ignore the error response
   *
   * @var string
   */
  protected $_ignoreError = true;

  /**
   * The API options
   *
   * @var array
   */
  protected $_options = array();

  /**
   * The API params
   *
   * @var array
   */
  protected $_params = array(
    'limit' => 25,
    'offset' => 1,
  );
  protected $_apiUrls = array(
    'search' => 'search',
    'trending' => 'trending'
  );
  protected $_testText = 'demo';

  /**
   * Constructor
   *
   * @param array $options
   */
  public function __construct($options = array())
  {
    $options['secretKey'] = Engine_Api::_()->getApi('settings', 'core')->getSetting('sitegifplayer.giphy.apikey', '');
    foreach( $options as $key => $value ) {
      $methodName = '_set' . ucfirst($key);
      if( method_exists($this, $methodName) ) {
        $this->$methodName($value);
        continue;
      }
      $this->_options[$key] = $value;
    }
    // Force the curl adapter if it's available
    if( extension_loaded('curl') ) {
      $adapter = new Zend_Http_Client_Adapter_Curl();
      $adapter->setCurlOption(CURLOPT_SSL_VERIFYPEER, false);
      $adapter->setCurlOption(CURLOPT_SSL_VERIFYHOST, false);
      $this->getHttpClient()->setAdapter($adapter);
    }
    $this->getHttpClient()->setConfig(array('timeout' => 15));
  }

  public function search($search, $options = array())
  {
    $options['q'] = $search;
    return $this->_convertResponse($this->call($this->_apiUrls['search'], $options));
  }

  public function trending($options = array())
  {
    return $this->_convertResponse($this->call($this->_apiUrls['trending'], $options));
  }

  public function call($apiUri, $options)
  {
    $cacheResponse = $this->_loadFromCache($apiUri, $options);
    if( $cacheResponse ) {
      return $cacheResponse;
    }
    $response = $this->_sendRequest($apiUri, $options);
    $this->_saveInCache($apiUri, $options, $response);
    return $response;
  }

  /**
   * Test the configuration
   * @param string $url
   *
   * @return self
   */
  public function test()
  {
    return $this->search($this->_testText, array(
        'limit' => 1
    ));
  }

  /**
   * Set the secret key base
   * @param string $key
   *
   * @return self
   */
  protected function _setSecretKey($key)
  {
    $this->_params['api_key'] = $key;
    return $this;
  }

  /**
   * Set the base url for vendor
   * @param string $baseUrl
   *
   * @return self
   */
  protected function _setBaseUrl($baseUrl)
  {
    if( empty($this->_baseUrl) ) {
      $this->_baseUrl = rtrim($baseUrl, '/');
    }
    return $this;
  }

  /**
   * Ignore error response
   * @param bool $ignoreError
   *
   * @return self
   */
  protected function _setIgnoreError($ignoreError)
  {
    $this->_ignoreError = $ignoreError;
    return $this;
  }

  /**
   * Send a client request
   * @param string $url
   *
   * @return array | null
   */
  protected function _sendRequest($apiUrl, $params)
  {
    // Send request
    $client = $this->_prepareHttpClient($apiUrl)
      ->setParameterGet($this->_getRequestParams($params));
    // Process response
    $response = $client->request();
    $responseData = $this->_processHttpResponse($response);
    return $responseData;
  }

  /**
   * Get the http client and set default parameters
   *
   * @return Zend_Http_Client
   */
  protected function _prepareHttpClient($apiUrl)
  {
    $uri = $this->_baseUrl . $apiUrl;
    return $this->getHttpClient()
        ->resetParameters()
        ->setMethod(Zend_Http_Client::GET)
        ->setHeaders('Accept', $this->_accept)
        ->setUri($uri)
    ;
  }

  /**
   * Process the response
   *
   * @param Zend_Http_Response $response
   * @return array
   * @throws Zend_Service_Exception
   */
  protected function _processHttpResponse($response)
  {
    // Check response body
    $responseData = $response->getBody();
    if( !is_string($responseData) || '' === $responseData ) {
      return !$this->_ignoreError ? array('error' => 'Invalid response') : '';
    }
    try {
      $responseData = Zend_Json::decode($responseData, Zend_Json::TYPE_ARRAY);
    } catch( Exception $e ) {
      $responseData = array('error' => 'Invalid json data or Invalid base url');
    }

    if( !empty($responseData['error']) && $this->_ignoreError ) {
      return;
    }

    return isset($responseData['data']) ? $responseData['data'] : $responseData;
  }

  protected function _convertResponse($response)
  {
    $results = array();
    foreach( $response as $row ) {
      $code = $row['id'];
      if( empty($row['images']['original']) ) {
        continue;
      }
      $results[] = array(
        'code' => $code,
        'title' => $row['slug'],
        'imgPath' => sprintf(Sitegifplayer_Api_Core::GIPHY_MEDIA_URL, $code)
      );
    }
    return $results;
  }

  protected function _getRequestParams($params = array())
  {

    return array_merge($this->_params, $params);
  }

  protected function _loadFromCache($apiUri, $params = array())
  {
    $hasCache = Zend_Registry::isRegistered('Zend_Cache') && Zend_Registry::get('Zend_Cache') instanceof Zend_Cache_Core;
    if( !$hasCache ) {
      return;
    }
    $cacheId = $this->_makeCacheId($apiUri, $params);
    $cache = Zend_Registry::get('Zend_Cache');
    $response = $cache->load($cacheId);
    return $response;
  }

  protected function _saveInCache($apiUri, $params = array(), $response = '')
  {
    if( !$response ) {
      return;
    }
    $cacheId = $this->_makeCacheId($apiUri, $params);
    $cache = Zend_Registry::get('Zend_Cache');
    $cache->save($response, $cacheId, array(), 3600);
  }

  protected function _makeCacheId($apiUri, $params = array())
  {
    return __CLASS__ . md5($apiUri . serialize($this->_getRequestParams($params)));
  }

}
